searchNodes=[{"doc":"Provides functions for interacting with a NATS Jetstream server.","ref":"Jetstream.html","title":"Jetstream","type":"module"},{"doc":"Sends AckAck acknowledgement to the server. Acknowledges a message was completely handled.","ref":"Jetstream.html#ack/1","title":"Jetstream.ack/1","type":"function"},{"doc":"Sends AckNext acknowledgement to the server. Acknowledges the message was handled and requests delivery of the next message to the reply subject. Only applies to Pull-mode.","ref":"Jetstream.html#ack_next/2","title":"Jetstream.ack_next/2","type":"function"},{"doc":"Sends AckProgress acknowledgement to the server. When sent before the AckWait period indicates that work is ongoing and the period should be extended by another equal to AckWait .","ref":"Jetstream.html#ack_progress/1","title":"Jetstream.ack_progress/1","type":"function"},{"doc":"Sends AckTerm acknowledgement to the server. Instructs the server to stop redelivery of a message without acknowledging it as successfully processed.","ref":"Jetstream.html#ack_term/1","title":"Jetstream.ack_term/1","type":"function"},{"doc":"Sends AckNak acknowledgement to the server. Signals that the message will not be processed now and processing can move onto the next message. NAK'd message will be retried.","ref":"Jetstream.html#nack/1","title":"Jetstream.nack/1","type":"function"},{"doc":"","ref":"Jetstream.html#t:message/0","title":"Jetstream.message/0","type":"type"},{"doc":"A module representing a NATS JetStream Consumer. Learn more about consumers: https://docs.nats.io/nats-concepts/jetstream/consumers The Jetstream.API.Consumer struct The struct's only mandatory field to set is the :stream_name . The rest will have the NATS default values set. Note that consumers are ephemeral by default. Set the :durable_name to make it durable. Consumer struct fields explanation: :stream_name - name of a stream the consumer is pointing at. :ack_policy - how the messages should be acknowledged. It has the following options: :explicit - the default policy. It means that each individual message must be acknowledged. It is the only allowed option for pull consumers. :none - no need to ack messages, the server will assume ack on delivery. :all - only the last received message needs to be acked, all the previous messages received are automatically acknowledged. :ack_wait - time in nanoseconds that server will wait for an ack for any individual. If an ack is not received in time, the message will be redelivered. :backoff - list of durations that represents a retry timescale for NAK'd messages or those being normally retried. :deliver_group - when set, will only deliver messages to subscriptions matching that group. :deliver_policy - specifies where in the stream it wants to start receiving messages. It has the following options: :all - the default policy. The consumer will start receiving from the earliest available message. :last - the consumer will start receiving messages with the last message added to the stream. :new - the consumer will only start receiving messages that were created after the customer was created. :by_start_sequence - the consumer is required to specify :opt_start_seq , the sequence number to start on. It will receive the closest available message moving forward in the sequence should the message specified have been removed based on the stream limit policy. :by_start_time - the consumer will start with messages on or after this time. The consumer is required to specify :opt_start_time , the time in the stream to start at. :last_per_subject - the consumer will start with the latest one for each filtered subject currently in the stream. :deliver_subject - the subject to deliver observed messages. Not allowed for pull subscriptions. A delivery subject is required for queue subscribing as it configures a subject that all the queue consumers should listen on. :description - a short description of the purpose of this customer. :durable_name - the name of the consumer, which the server will track, allowing resuming consumption where left off. By default, a consumer is ephemeral. To make the consumer durable, set the name. See naming . :filter_subject - when consuming from a stream with a wildcard subject, this allows you to select a subset of the full wildcard subject to receive messages from. :flow_control - when set to true, an empty message with Status header 100 and a reply subject will be sent. Consumers must reply to these messages to control the rate of message delivery. :headers_only - delivers only the headers of messages in the stream and not the bodies. Additionally adds the Nats-Msg-Size header to indicate the size of the removed payload. :idle_heartbeat - if set, the server will regularly send a status message to the client while there are no new messages to send. This lets the client know that the JetStream service is still up and running, even when there is no activity on the stream. The message status header will have a code of 100. Unlike :flow_control , it will have no reply to address. It may have a description like &quot;Idle Heartbeat&quot;. :inactive_threshold - duration that instructs the server to clean up ephemeral consumers that are inactive for that long. :max_ack_pending - it sets the maximum number of messages without an acknowledgement that can be outstanding, once this limit is reached, message delivery will be suspended. It cannot be used with :ack_none ack policy. This maximum number of pending acks applies for all the consumer's subscriber processes. A value of -1 means there can be any number of pending acks (i.e. no flow control). :max_batch - the largest batch property that may be specified when doing a pull on a Pull consumer. :max_deliver - the maximum number of times a specific message will be delivered. Applies to any message that is re-sent due to ack policy. :max_expires - the maximum expires value that may be set when doing a pull on a Pull consumer. :max_waiting - the number of pulls that can be outstanding on a pull consumer, pulls received after this is reached are ignored. :opt_start_seq - use with :deliver_policy set to :by_start_sequence . It represents the sequence number to start consuming on. :opt_start_time - use with :deliver_policy set to :by_start_time . It represents the time to start consuming at. :rate_limit_bps - used to throttle the delivery of messages to the consumer, in bits per second. :replay_policy - it applies when the :deliver_policy is set to :all , :by_start_sequence or :by_start_time . It has the following options: :instant - the default policy. The messages will be pushed to the client as fast as possible. :original - the messages in the stream will be pushed to the client at the same rate that they were originally received. :sample_freq - Sets the percentage of acknowledgements that should be sampled for observability, 0-100. This value is a binary and for example allows both 30 and 30% as valid values.","ref":"Jetstream.API.Consumer.html","title":"Jetstream.API.Consumer","type":"module"},{"doc":"Creates a consumer. When consumer's :durable_name field is not set, the function creates an ephemeral consumer. Otherwise, it creates a durable consumer. Examples iex&gt; { :ok , _response } = Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } ) iex&gt; { :ok , %{ name : &quot;consumer&quot; , stream_name : &quot;stream&quot; } } = Jetstream.API.Consumer . create ( :gnat , % Jetstream.API.Consumer { durable_name : &quot;consumer&quot; , stream_name : &quot;stream&quot; } ) iex&gt; { :ok , _response } = Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } ) iex&gt; { :error , %{ &quot;description&quot; =&gt; &quot;consumer delivery policy is deliver by start sequence, but optional start sequence is not set&quot; } } = Jetstream.API.Consumer . create ( :gnat , % Jetstream.API.Consumer { durable_name : &quot;consumer&quot; , stream_name : &quot;stream&quot; , deliver_policy : :by_start_sequence } )","ref":"Jetstream.API.Consumer.html#create/2","title":"Jetstream.API.Consumer.create/2","type":"function"},{"doc":"Deletes a consumer. Examples iex&gt; { :ok , _response } = Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } ) iex&gt; { :ok , _response } = Jetstream.API.Consumer . create ( :gnat , % Jetstream.API.Consumer { durable_name : &quot;consumer&quot; , stream_name : &quot;stream&quot; } ) iex&gt; Jetstream.API.Consumer . delete ( :gnat , &quot;stream&quot; , &quot;consumer&quot; ) :ok iex&gt; { :error , %{ &quot;code&quot; =&gt; 404 , &quot;description&quot; =&gt; &quot;stream not found&quot; } } = Jetstream.API.Consumer . delete ( :gnat , &quot;wrong_stream&quot; , &quot;consumer&quot; )","ref":"Jetstream.API.Consumer.html#delete/3","title":"Jetstream.API.Consumer.delete/3","type":"function"},{"doc":"Information about the consumer. Examples iex&gt; { :ok , _response } = Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } ) iex&gt; { :ok , _response } = Jetstream.API.Consumer . create ( :gnat , % Jetstream.API.Consumer { durable_name : &quot;consumer&quot; , stream_name : &quot;stream&quot; } ) iex&gt; { :ok , %{ created : _ } } = Jetstream.API.Consumer . info ( :gnat , &quot;stream&quot; , &quot;consumer&quot; ) iex&gt; { :error , %{ &quot;code&quot; =&gt; 404 , &quot;description&quot; =&gt; &quot;stream not found&quot; } } = Jetstream.API.Consumer . info ( :gnat , &quot;wrong_stream&quot; , &quot;consumer&quot; )","ref":"Jetstream.API.Consumer.html#info/3","title":"Jetstream.API.Consumer.info/3","type":"function"},{"doc":"Paged list of known consumers, including their current info. Examples iex&gt; { :ok , _response } = Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } ) iex&gt; { :ok , %{ consumers : _ , limit : 1024 , offset : 0 , total : _ } } = Jetstream.API.Consumer . list ( :gnat , &quot;stream&quot; ) iex&gt; { :error , %{ &quot;code&quot; =&gt; 404 , &quot;description&quot; =&gt; &quot;stream not found&quot; } } = Jetstream.API.Consumer . list ( :gnat , &quot;wrong_stream&quot; )","ref":"Jetstream.API.Consumer.html#list/3","title":"Jetstream.API.Consumer.list/3","type":"function"},{"doc":"Requests a next message from a stream to be consumed. The response (consumed message)will be sent on the subject given as the reply_to parameter. Options batch - How many messages to receive. Messages will be sent to the reply_to subject separately. Defaults to 1. expires - Time in nanoseconds the request will be kept in the server. Once this time passes a message with empty body and topic set to reply_to subject is sent. Useful when polling the server frequently and not wanting the pull requests to accumulate. By default, the pull request stays in the server until a message comes. no_wait - Boolean value which indicates whether the pull request should be accumulated on the server. When set to true and no message is present to be consumed, a message with empty body and topic value set to reply_to is sent. Defaults to false. Example iex&gt; { :ok , _response } = Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } ) iex&gt; { :ok , _response } = Jetstream.API.Consumer . create ( :gnat , % Jetstream.API.Consumer { durable_name : &quot;consumer&quot; , stream_name : &quot;stream&quot; } ) iex&gt; { :ok , _sid } = Gnat . sub ( :gnat , self ( ) , &quot;reply_subject&quot; ) iex&gt; :ok = Jetstream.API.Consumer . request_next_message ( :gnat , &quot;stream&quot; , &quot;consumer&quot; , &quot;reply_subject&quot; ) iex&gt; :ok = Gnat . pub ( :gnat , &quot;subject&quot; , &quot;message1&quot; ) iex&gt; assert_receive { :msg , %{ body : &quot;message1&quot; , topic : &quot;subject&quot; } }","ref":"Jetstream.API.Consumer.html#request_next_message/5","title":"Jetstream.API.Consumer.request_next_message/5","type":"function"},{"doc":"","ref":"Jetstream.API.Consumer.html#t:config/0","title":"Jetstream.API.Consumer.config/0","type":"type"},{"doc":"","ref":"Jetstream.API.Consumer.html#t:consumers/0","title":"Jetstream.API.Consumer.consumers/0","type":"type"},{"doc":"","ref":"Jetstream.API.Consumer.html#t:info/0","title":"Jetstream.API.Consumer.info/0","type":"type"},{"doc":"","ref":"Jetstream.API.Consumer.html#t:t/0","title":"Jetstream.API.Consumer.t/0","type":"type"},{"doc":"API for interacting with the Key/Value store functionality in Nats Jetstream. Learn about the Key/Value store: https://docs.nats.io/nats-concepts/jetstream/key-value-store","ref":"Jetstream.API.KV.html","title":"Jetstream.API.KV","type":"module"},{"doc":"Get all the non-deleted key-value pairs for a Bucket Examples iex&gt; { :ok , %{ &quot;key1&quot; =&gt; &quot;value1&quot; } } = Jetstream.API.KV . contents ( :gnat , &quot;my_bucket&quot; )","ref":"Jetstream.API.KV.html#contents/2","title":"Jetstream.API.KV.contents/2","type":"function"},{"doc":"Create a new Key/Value bucket. Can include the following options :history - How many historic values to keep per key (defaults to 1, max of 64) :ttl - How long to keep values for (in nanoseconds) :max_bucket_size - The max number of bytes the bucket can hold :max_value_size - The max number of bytes a value may be :description - A description for the bucket :replicas - How many replicas of the data to store :storage - Storage backend to use (:file, :memory) :placement - A map with :cluster (required) and :tags (optional) Examples iex&gt;{:ok, info} = Jetstream.API.KV.create_bucket(:gnat, &quot;my_bucket&quot;)","ref":"Jetstream.API.KV.html#create_bucket/3","title":"Jetstream.API.KV.create_bucket/3","type":"function"},{"doc":"Create a Key in a Key/Value Bucket Examples iex&gt; :ok = Jetstream.API.KV . create_key ( :gnat , &quot;my_bucket&quot; , &quot;my_key&quot; , &quot;my_value&quot; )","ref":"Jetstream.API.KV.html#create_key/5","title":"Jetstream.API.KV.create_key/5","type":"function"},{"doc":"Delete a Key/Value bucket Examples iex&gt;:ok = Jetstream.API.KV.delete_bucket(:gnat, &quot;my_bucket&quot;)","ref":"Jetstream.API.KV.html#delete_bucket/2","title":"Jetstream.API.KV.delete_bucket/2","type":"function"},{"doc":"Delete a Key from a K/V Bucket Examples iex&gt; :ok = Jetstream.API.KV . delete_key ( :gnat , &quot;my_bucket&quot; , &quot;my_key&quot; )","ref":"Jetstream.API.KV.html#delete_key/4","title":"Jetstream.API.KV.delete_key/4","type":"function"},{"doc":"Get the value for a key in a particular K/V bucket Examples iex&gt; &quot;my_value&quot; = Jetstream.API.KV . get_value ( :gnat , &quot;my_bucket&quot; , &quot;my_key&quot; )","ref":"Jetstream.API.KV.html#get_value/3","title":"Jetstream.API.KV.get_value/3","type":"function"},{"doc":"Purge a Key from a K/V bucket. This will remove any revision history the key had Examples iex&gt; :ok = Jetstream.API.KV . purge_key ( :gnat , &quot;my_bucket&quot; , &quot;my_key&quot; )","ref":"Jetstream.API.KV.html#purge_key/4","title":"Jetstream.API.KV.purge_key/4","type":"function"},{"doc":"Put a value into a Key in a K/V Bucket Examples iex&gt; :ok = Jetstream.API.KV . put_value ( :gnat , &quot;my_bucket&quot; , &quot;my_key&quot; , &quot;my_value&quot; )","ref":"Jetstream.API.KV.html#put_value/5","title":"Jetstream.API.KV.put_value/5","type":"function"},{"doc":"Stops a previously running monitor. This will unsubscribe from the key changes and remove the ephemeral consumer Examples iex&gt; :ok = Jetstream.API.KV . unwatch ( pid )","ref":"Jetstream.API.KV.html#unwatch/1","title":"Jetstream.API.KV.unwatch/1","type":"function"},{"doc":"Starts a monitor for key changes in a given bucket. Supply a handler that will receive key change notifications. Examples iex&gt; { :ok , _pid } = Jetstream.API.KV . watch ( :gnat , &quot;my_bucket&quot; , fn action , key , value -&gt; IO . puts ( &quot; \#{ action } taken on \#{ key } &quot; ) end )","ref":"Jetstream.API.KV.html#watch/3","title":"Jetstream.API.KV.watch/3","type":"function"},{"doc":"","ref":"Jetstream.API.KV.html#t:bucket_options/0","title":"Jetstream.API.KV.bucket_options/0","type":"type"},{"doc":"The watcher server establishes a subscription to the changes that occur to a given key-value bucket. The consumer-supplied handler function will be sent an indicator as to whether the change is a delete or an add, as well as the key being changed and the value (if it was added). Ensure that you call stop with a watcher pid when you no longer need to be notified about key changes","ref":"Jetstream.API.KV.Watcher.html","title":"Jetstream.API.KV.Watcher","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Jetstream.API.KV.Watcher.html#child_spec/1","title":"Jetstream.API.KV.Watcher.child_spec/1","type":"function"},{"doc":"Callback implementation for GenServer.init/1 .","ref":"Jetstream.API.KV.Watcher.html#init/1","title":"Jetstream.API.KV.Watcher.init/1","type":"function"},{"doc":"","ref":"Jetstream.API.KV.Watcher.html#start_link/1","title":"Jetstream.API.KV.Watcher.start_link/1","type":"function"},{"doc":"","ref":"Jetstream.API.KV.Watcher.html#stop/1","title":"Jetstream.API.KV.Watcher.stop/1","type":"function"},{"doc":"","ref":"Jetstream.API.KV.Watcher.html#t:keywatch_handler/0","title":"Jetstream.API.KV.Watcher.keywatch_handler/0","type":"type"},{"doc":"","ref":"Jetstream.API.KV.Watcher.html#t:watcher_options/0","title":"Jetstream.API.KV.Watcher.watcher_options/0","type":"type"},{"doc":"API for interacting with the JetStream Object Store Learn more about Object Store: https://docs.nats.io/nats-concepts/jetstream/obj_store","ref":"Jetstream.API.Object.html","title":"Jetstream.API.Object","type":"module"},{"doc":"","ref":"Jetstream.API.Object.html#create_bucket/3","title":"Jetstream.API.Object.create_bucket/3","type":"function"},{"doc":"","ref":"Jetstream.API.Object.html#delete_bucket/2","title":"Jetstream.API.Object.delete_bucket/2","type":"function"},{"doc":"","ref":"Jetstream.API.Object.html#delete_object/3","title":"Jetstream.API.Object.delete_object/3","type":"function"},{"doc":"","ref":"Jetstream.API.Object.html#get_object/4","title":"Jetstream.API.Object.get_object/4","type":"function"},{"doc":"","ref":"Jetstream.API.Object.html#info/3","title":"Jetstream.API.Object.info/3","type":"function"},{"doc":"","ref":"Jetstream.API.Object.html#list_objects/3","title":"Jetstream.API.Object.list_objects/3","type":"function"},{"doc":"","ref":"Jetstream.API.Object.html#put_object/4","title":"Jetstream.API.Object.put_object/4","type":"function"},{"doc":"","ref":"Jetstream.API.Object.Meta.html","title":"Jetstream.API.Object.Meta","type":"module"},{"doc":"A module representing a NATS JetStream Stream. Learn more about Streams: https://docs.nats.io/nats-concepts/jetstream/streams The Jetstream.API.Stream struct The struct's mandatory fields are :name and :subjects . The rest will have the NATS default values set. Stream struct fields explanation: :allow_rollup_hdrs - allows the use of the Nats-Rollup header to replace all contents of a stream, or subject in a stream, with a single new message. :deny_delete - restricts the ability to delete messages from a stream via the API. Cannot be changed once set to true. :deny_purge - restricts the ability to purge messages from a stream via the API. Cannot be change once set to true. :description - a short description of the purpose of this stream. :discard - determines what happens when a Stream reaches its limits. It has the following options: :old - the default option. Old messages are deleted. :new - refuses new messages. :duplicate_window - the window within which to track duplicate messages, expressed in nanoseconds. :max_age - maximum age of any message in the Stream, expressed in nanoseconds. :max_bytes - how many bytes the Stream may contain. Adheres to :discard , removing oldest or refusing new messages if the Stream exceeds this size. :max_consumers - how many Consumers can be defined for a given Stream, -1 for unlimited. :max_msg_size - the largest message that will be accepted by the Stream. :max_msgs_per_subject - For wildcard streams ensure that for every unique subject this many messages are kept - a per subject retention limit. Only available on nats-server versions greater than 2.3.0 :max_msgs - how many messages may be in a Stream. Adheres to :discard , removing oldest or refusing new messages if the Stream exceeds this number of messages :mirror - maintains a 1:1 mirror of another stream with name matching this property. When a mirror is configured subjects and sources must be empty. :name - a name for the Stream. See naming . :no_ack - disables acknowledging messages that are received by the Stream. :num_replicas - how many replicas to keep for each message. :placement - placement directives to consider when placing replicas of this stream, random placement when unset. It has the following properties: :cluster - the desired cluster name to place the stream. :tags - tags required on servers hosting this stream. :retention - how messages are retained in the Stream. Once this is exceeded, old messages are removed. It has the following options: :limits - the default policy. :interest :workqueue :sealed - sealed streams do not allow messages to be deleted via limits or API, sealed streams can not be unsealed via configuration update. Can only be set on already created streams via the Update API. :sources - list of stream names to replicate into this stream. :storage - the type of storage backend. Available options: :file :memory :subjects - a list of subjects to consume, supports wildcards. :template_owner - when the Stream is managed by a Stream Template this identifies the template that manages the Stream.","ref":"Jetstream.API.Stream.html","title":"Jetstream.API.Stream","type":"module"},{"doc":"Creates a new Stream. Examples iex&gt; { :ok , %{ created : _ } } = Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } )","ref":"Jetstream.API.Stream.html#create/2","title":"Jetstream.API.Stream.create/2","type":"function"},{"doc":"Deletes a Stream and all its data. Examples iex&gt; Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } ) iex&gt; Jetstream.API.Stream . delete ( :gnat , &quot;stream&quot; ) :ok iex&gt; { :error , %{ &quot;code&quot; =&gt; 404 , &quot;description&quot; =&gt; &quot;stream not found&quot; } } = Jetstream.API.Stream . delete ( :gnat , &quot;wrong_stream&quot; )","ref":"Jetstream.API.Stream.html#delete/2","title":"Jetstream.API.Stream.delete/2","type":"function"},{"doc":"Get a message from the stream either by &quot;stream sequence number&quot; or the &quot;last message for a given subject&quot;","ref":"Jetstream.API.Stream.html#get_message/3","title":"Jetstream.API.Stream.get_message/3","type":"function"},{"doc":"Information about config and state of a Stream. Examples iex&gt; Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } ) iex&gt; { :ok , %{ created : _ } } = Jetstream.API.Stream . info ( :gnat , &quot;stream&quot; ) iex&gt; { :error , %{ &quot;code&quot; =&gt; 404 , &quot;description&quot; =&gt; &quot;stream not found&quot; } } = Jetstream.API.Stream . info ( :gnat , &quot;wrong_stream&quot; )","ref":"Jetstream.API.Stream.html#info/2","title":"Jetstream.API.Stream.info/2","type":"function"},{"doc":"Paged list of known Streams including all their current information. Options :offset - Number of records to skip :subject - A subject the Stream must collect to appear in the list. Examples iex&gt; { :ok , %{ total : _ , offset : 0 , limit : 1024 , streams : _ } } = Jetstream.API.Stream . list ( :gnat )","ref":"Jetstream.API.Stream.html#list/2","title":"Jetstream.API.Stream.list/2","type":"function"},{"doc":"Purges all of data in the stream but doesn't delete the stream. Examples iex&gt; Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;subject&quot; ] } ) iex&gt; Jetstream.API.Stream . purge ( :gnat , &quot;stream&quot; ) :ok iex&gt; { :error , %{ &quot;code&quot; =&gt; 404 , &quot;description&quot; =&gt; &quot;stream not found&quot; } } = Jetstream.API.Stream . purge ( :gnat , &quot;wrong_stream&quot; )","ref":"Jetstream.API.Stream.html#purge/2","title":"Jetstream.API.Stream.purge/2","type":"function"},{"doc":"Purges some of the messages in a stream. Examples iex&gt; Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;stream&quot; , subjects : [ &quot;sub1&quot; , &quot;sub2&quot; ] } ) iex&gt; Jetstream.API.Stream . purge ( :gnat , &quot;stream&quot; , %{ filter : &quot;sub1&quot; } ) :ok","ref":"Jetstream.API.Stream.html#purge/3","title":"Jetstream.API.Stream.purge/3","type":"function"},{"doc":"Updates a Stream. Examples iex&gt; { :ok , %{ created : _ } } = Jetstream.API.Stream . create ( :gnat , % Jetstream.API.Stream { name : &quot;update_test_stream&quot; , subjects : [ &quot;update_subject&quot; ] } ) iex&gt; { :ok , _ } = Jetstream.API.Stream . update ( :gnat , % Jetstream.API.Stream { name : &quot;update_test_stream&quot; , subjects : [ &quot;update_subject&quot; , &quot;new.update_subject&quot; ] } )","ref":"Jetstream.API.Stream.html#update/2","title":"Jetstream.API.Stream.update/2","type":"function"},{"doc":"","ref":"Jetstream.API.Stream.html#t:info/0","title":"Jetstream.API.Stream.info/0","type":"type"},{"doc":"seq - Stream sequence number of the message to retrieve, cannot be combined with last_by_subj last_by_subj - Retrieves the last message for a given subject, cannot be combined with seq","ref":"Jetstream.API.Stream.html#t:message_access_method/0","title":"Jetstream.API.Stream.message_access_method/0","type":"type"},{"doc":"data - The decoded message payload subject - The subject the message was originally received on time - The time the message was received seq - The sequence number of the message in the Stream hdrs - The decoded headers for the message","ref":"Jetstream.API.Stream.html#t:message_response/0","title":"Jetstream.API.Stream.message_response/0","type":"type"},{"doc":"","ref":"Jetstream.API.Stream.html#t:nanoseconds/0","title":"Jetstream.API.Stream.nanoseconds/0","type":"type"},{"doc":"","ref":"Jetstream.API.Stream.html#t:placement/0","title":"Jetstream.API.Stream.placement/0","type":"type"},{"doc":"code - HTTP like error code in the 300 to 500 range description - A human friendly description of the error err_code - The NATS error code unique to each kind of error","ref":"Jetstream.API.Stream.html#t:response_error/0","title":"Jetstream.API.Stream.response_error/0","type":"type"},{"doc":"Stream source fields explained: :name - stream name. :opt_start_seq - sequence to start replicating from. :opt_start_time - timestamp to start replicating from. :filter_subject - replicate only a subset of messages based on filter. :external - configuration referencing a stream source in another account or JetStream domain. It has the following parameters: :api - the subject prefix that imports other account/domain $JS.API.CONSUMER.&gt; subjects :deliver - the delivery subject to use for push consumer","ref":"Jetstream.API.Stream.html#t:source/0","title":"Jetstream.API.Stream.source/0","type":"type"},{"doc":"","ref":"Jetstream.API.Stream.html#t:source_info/0","title":"Jetstream.API.Stream.source_info/0","type":"type"},{"doc":"","ref":"Jetstream.API.Stream.html#t:state/0","title":"Jetstream.API.Stream.state/0","type":"type"},{"doc":"","ref":"Jetstream.API.Stream.html#t:streams/0","title":"Jetstream.API.Stream.streams/0","type":"type"},{"doc":"","ref":"Jetstream.API.Stream.html#t:t/0","title":"Jetstream.API.Stream.t/0","type":"type"},{"doc":"A behaviour which provides the NATS JetStream Pull Consumer functionalities. When a Consumer is pull-based, it means that the messages will be delivered when the server is asked for them. Example Declare a module which uses Jetstream.PullConsumer and implements init/1 and handle_message/2 callbacks. defmodule MyApp.PullConsumer do use Jetstream.PullConsumer def start_link ( arg ) do Jetstream.PullConsumer . start_link ( __MODULE__ , arg ) end @impl true def init ( _arg ) do { :ok , nil , connection_name : :gnat , stream_name : &quot;TEST_STREAM&quot; , consumer_name : &quot;TEST_CONSUMER&quot; } end @impl true def handle_message ( message , state ) do # Do some processing with the message. { :ack , state } end end You can then place your Pull Consumer in a supervision tree. Remember that you need to have the Gnat.ConnectionSupervisor set up. defmodule MyApp.Application do use Application @impl true def start ( _type , _args ) do children = [ # Create NATS connection { Gnat.ConnectionSupervisor , ... } , # Start NATS Jetstream Pull Consumer MyApp.PullConsumer , ] opts = [ strategy : :one_for_one ] Supervisor . start_link ( children , opts ) end end Connection Options In order to establish consumer connection with NATS, you need to pass several connection options via keyword list in third element of a tuple returned from init/1 callback. Following options must be provided. Omitting this options will cause the process to raise errors upon initialization: :connection_name - Gnat connection or Gnat.ConnectionSupervisor name/PID. :stream_name - name of an existing string the consumer will consume messages from. :consumer_name - name of an existing consumer pointing at the stream. You can also pass the optional ones: :connection_retry_timeout - a duration in milliseconds after which the PullConsumer which failed to establish NATS connection retries, defaults to 1000 :connection_retries - a number of attempts the PullConsumer will make to establish the NATS connection. When this value is exceeded, the pull consumer stops with the :timeout reason, defaults to 10 :inbox_prefix - allows the default _INBOX. prefix to be customized. Should end with a dot. Dynamic Connection Options It is possible that you have to determine some of the options dynamically depending on pull consumer's init argument. To do so, it is recommended to derive these options values from some init argument: defmodule MyApp.PullConsumer do use Jetstream.PullConsumer def start_link ( ) do Jetstream.PullConsumer . start_link ( __MODULE__ , %{ counter : counter } ) end @impl true def init ( %{ counter : counter } ) do { :ok , nil , connection_name : :gnat , stream_name : &quot;TEST_STREAM_ \#{ counter } &quot; , consumer_name : &quot;TEST_CONSUMER_ \#{ counter } &quot; } end ... end How to supervise A PullConsumer is most commonly started under a supervision tree. When we invoke use Jetstream.PullConsumer , it automatically defines a child_spec/1 function that allows us to start the pull consumer directly under a supervisor. To start a pull consumer under a supervisor with an initial argument of :example, one may do: children = [ { MyPullConsumer , :example } ] Supervisor . start_link ( children , strategy : :one_for_all ) While one could also simply pass the MyPullConsumer as a child to the supervisor, such as: children = [ MyPullConsumer # Same as {MyPullConsumer, []} ] Supervisor . start_link ( children , strategy : :one_for_all ) A common approach is to use a keyword list, which allows setting init argument and server options, for example: def start_link ( opts ) do { initial_state , opts } = Keyword . pop ( opts , :initial_state , nil ) Jetstream.PullConsumer . start_link ( __MODULE__ , initial_state , opts ) end and then you can use MyPullConsumer , {MyPullConsumer, name: :my_consumer} or even {MyPullConsumer, initial_state: :example, name: :my_consumer} as a child specification. use Jetstream.PullConsumer also accepts a list of options which configures the child specification and therefore how it runs under a supervisor. The generated child_spec/1 can be customized with the following options: :id - the child specification identifier, defaults to the current module :restart - when the child should be restarted, defaults to :permanent :shutdown - how to shut down the child, either immediately or by giving it time to shut down For example: use Jetstream.PullConsumer , restart : :transient , shutdown : 10_000 See the &quot;Child specification&quot; section in the Supervisor module for more detailed information. The @doc annotation immediately preceding use Jetstream.PullConsumer will be attached to the generated child_spec/1 function. Name registration A pull consumer is bound to the same name registration rules as GenServers. Read more about it in the GenServer documentation.","ref":"Jetstream.PullConsumer.html","title":"Jetstream.PullConsumer","type":"behaviour"},{"doc":"Closes the pull consumer and stops underlying process. Example { :ok , consumer } = PullConsumer . start_link ( ExamplePullConsumer , connection_name : :gnat , stream_name : &quot;TEST_STREAM&quot; , consumer_name : &quot;TEST_CONSUMER&quot; ) :ok = PullConsumer . close ( consumer )","ref":"Jetstream.PullConsumer.html#close/1","title":"Jetstream.PullConsumer.close/1","type":"function"},{"doc":"Invoked to synchronously process a message pulled by the consumer. Depending on the value it returns, the acknowledgement is or is not sent. ACK actions Possible ACK actions values explained: :ack - acknowledges the message was handled and requests delivery of the next message to the reply subject. :nack - signals that the message will not be processed now and processing can move onto the next message, NAK'd message will be retried. :term - instructs the server to stop redelivery of a message without acknowledging it as successfully processed. :noreply - nothing is sent. You may send later asynchronously an ACK or NACK message using the Jetstream.ack/1 or Jetstream.nack/1 and similar functions from Jetstream module. Example def handle_message ( message , state ) do IO . inspect ( message ) { :ack , state } end","ref":"Jetstream.PullConsumer.html#c:handle_message/2","title":"Jetstream.PullConsumer.handle_message/2","type":"callback"},{"doc":"Invoked when the server is started. start_link/3 or start/3 will block until it returns. init_arg is the argument term (second argument) passed to start_link/3 . See Connection.init/1 for more details.","ref":"Jetstream.PullConsumer.html#c:init/1","title":"Jetstream.PullConsumer.init/1","type":"callback"},{"doc":"Starts a Jetstream.PullConsumer process without links (outside of a supervision tree). See start_link/3 for more information.","ref":"Jetstream.PullConsumer.html#start/3","title":"Jetstream.PullConsumer.start/3","type":"function"},{"doc":"Starts a pull consumer linked to the current process with the given function. This is often used to start the pull consumer as part of a supervision tree. Once the server is started, the init/1 function of the given module is called with init_arg as its argument to initialize the server. To ensure a synchronized start-up procedure, this function does not return until init/1 has returned. See GenServer.start_link/3 for more details.","ref":"Jetstream.PullConsumer.html#start_link/3","title":"Jetstream.PullConsumer.start_link/3","type":"function"},{"doc":"Connection option values used to connect the consumer to NATS server.","ref":"Jetstream.PullConsumer.html#t:connection_option/0","title":"Jetstream.PullConsumer.connection_option/0","type":"type"},{"doc":"Connection options used to connect the consumer to NATS server.","ref":"Jetstream.PullConsumer.html#t:connection_options/0","title":"Jetstream.PullConsumer.connection_options/0","type":"type"},{"doc":"The pull consumer reference.","ref":"Jetstream.PullConsumer.html#t:consumer/0","title":"Jetstream.PullConsumer.consumer/0","type":"type"},{"doc":"A GenStage producer meant to work with Broadway . It continuously receives messages from a NATS JetStream and acknowledges them after being successfully processed. Options Connection options The following options are mandatory. connection_name - The name of Gnat process or Gnat connection supervisor. stream_name - The name of stream to consume from. consumer_name - The name of consumer. Optional options: connection_retry_timeout - time in milliseconds, after which the failing connection will retry. Defaults to 1000 . connection_retries - number of failing connection retries the producer will make before shutting down. Defaults to 10 . inbox_prefix - custom prefix for listening topic. Defaults to _INBOX. . Message pulling options receive_interval - The duration in milliseconds for which the producer waits before making a request for more messages. Defaults to 5000 . receive_timeout - The maximum time to wait for NATS Jetstream to respond with a requested message. Defaults to :infinity . Acknowledger options These options are passed to the acknowledger on its initialization. :on_success - Configures the behaviour for successful messages. Defaults to :ack . :on_failure - Configures the behaviour for failed messages. Defaults to :nack . Acknowledgements By default, successful messages are acknowledged, and failed ones are nack'ed. You can change this behaviour by setting the :on_success or/and :on_failure options. You can also change the acknowledgement action for individual messages using the Broadway.Message.configure_ack/2 function. The supported options are: :ack - Acknowledges a message was completely handled. :nack - Signals that the message will not be processed now and will be redelivered. :term - Tells the server to stop redelivery of a message without acknowledging it. Example Example Broadway module definition: defmodule MyBroadway do use Broadway def start_link ( _opts ) do Broadway . start_link ( __MODULE__ , name : MyBroadway , producer : [ module : { OffBroadway.Jetstream.Producer , connection_name : :gnat , stream_name : &quot;TEST_STREAM&quot; , consumer_name : &quot;TEST_CONSUMER&quot; } , concurrency : 10 ] , processors : [ default : [ concurrency : 10 ] ] , batchers : [ example : [ concurrency : 5 , batch_size : 10 , batch_timeout : 2_000 ] ] ) end def handle_message ( _processor_name , message , _context ) do message |&gt; Message . update_data ( &amp; process_data / 1 ) |&gt; Message . put_batcher ( :example ) end defp process_data ( data ) do # Some data processing end def handle_batch ( :example , messages , _batch_info , _context ) do # Do something with batch messages end end Learn more about available options in Broadway documentation . Once you have your Broadway pipeline defined, you can add it to your supervision tree: children = [ { MyBroadway , [ ] } ] Supervisor . start_link ( children , strategy : :one_for_one )","ref":"OffBroadway.Jetstream.Producer.html","title":"OffBroadway.Jetstream.Producer","type":"module"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Makes KV.put_value , KV.create_key , KV.delete_key , and KV.purge_key synchronous","ref":"changelog.html#0-0-7","title":"Changelog - 0.0.7","type":"extras"},{"doc":"Adds KV watcher https://github.com/mmmries/jetstream/pull/72","ref":"changelog.html#0-0-6","title":"Changelog - 0.0.6","type":"extras"},{"doc":"Include NATS headers in Broadway.Message metadata https://github.com/mmmries/jetstream/pull/68","ref":"changelog.html#0-0-5","title":"Changelog - 0.0.5","type":"extras"},{"doc":"fix bug with Jetstream.API.KV.contents/2 where consumer names sometimes had / characters. https://github.com/mmmries/jetstream/pull/66","ref":"changelog.html#0-0-4","title":"Changelog - 0.0.4","type":"extras"},{"doc":"add Jetstream.API.KV.contents/2 function https://github.com/mmmries/jetstream/pull/64","ref":"changelog.html#0-0-3","title":"Changelog - 0.0.3","type":"extras"},{"doc":"Jetstream is a distributed persistence system built-in to NATS . It provides a streaming system that lets you capture streams of events from various sources and persist these into persistent stores, which you can immediately or later replay for processing. This library exposes interfaces for publishing, consuming and managing Jetstream services. It builds on top of Gnat , the officially supported Elixir client for NATS. Let's get Jetstream up and running Using Broadway with Jetstream Pull Consumer API Create, update and delete Jetstream streams and consumers via Elixir","ref":"overview.html","title":"Overview","type":"extras"},{"doc":"In this guide, we're going to learn how to install Jetstream in your project and start consuming messages from your streams.","ref":"getting_started.html","title":"Getting Started","type":"extras"},{"doc":"The following Docker Compose file will do the job: version: &quot;3&quot; services: nats: image: nats:latest command: - -js ports: - 4222:4222 Save this snippet as docker-compose.yml and run the following command: docker compose up -d Let's also create Jetstream stream where we will publish our hello world messages: nats stream add HELLO --subjects=&quot;greetings&quot; Tip You can also manage Jetstream streams and consumers via Elixir. You can see more details in this guide .","ref":"getting_started.html#starting-jetstream","title":"Getting Started - Starting Jetstream","type":"extras"},{"doc":"To start off with, we'll generate a new Elixir application by running this command: mix new hello_jetstream -- sup We need to have a supervision tree up and running in your app, and the --sup option ensures that. To add Jetstream to this application, you need to add Jetstream and Gnat libraries to your deps definition in our mix.exs file. Fill exact version requirements from each package Hex.pm pages. defp deps do [ { :gnat , ... } , { :jetstream , ... } ] end To install these dependencies, we will run this command: mix deps.get Now let's connect to our NATS server. To do this, you need to start Gnat.ConnectionSupervisor under our application's supervision tree. Add following to lib/hello_jetstream/application.ex : def start ( _type , _args ) do children = [ ... # Create NATS connection { Gnat.ConnectionSupervisor , %{ name : :gnat , connection_settings : [ %{ host : &quot;localhost&quot; , port : 4222 } ] } } , ] ... This piece of configuration will start Gnat processes that connect to the NATS server and allow publishing and subscribing to any subjects. Jetstream operates using plain NATS subjects which follow specific naming and message format conventions. Let's now create a pull consumer which will subscribe a specific Jetstream stream and print incoming messages to standard output.","ref":"getting_started.html#adding-jetstream-and-gnat-to-an-application","title":"Getting Started - Adding Jetstream and Gnat to an application","type":"extras"},{"doc":"Jetstream requires us to allocate a view/cursor of the stream that our consumer will operate on. In Jetstream terminology, this view is called a consumer (Funnily enough we've just implemented a consumer in our code, coincidence?). Jetstream documentation offers great insights on benefits of having this separate concept so we won't duplicate work here. Jetstream offers two stream consuming modes: push and pull . In push mode, Jetstream will simply send messages to selected consumers immediately when they are received. This approach does offer congestion control, so it is not recommended for high-volume and/or reliability sensitive streams. You do not really need this library to implement push consumer because all building blocks are in Gnat library. You can read more about push consumers in this guide . On the other hand, in pull mode consumers ask Jetstream for more messages when they are ready to process them. This is the recommended approach for most use cases and we will proceed with it in this guide. This is just a brief outline For more details about differences between consumer modes, consult Jetstream documentation . Let's create a pull consumer module within our application at lib/hello_jetstream/logger_pull_consumer.ex : defmodule HelloJetstream.LoggerPullConsumer do use Jetstream.PullConsumer def start_link ( [ ] ) do Jetstream.PullConsumer . start_link ( __MODULE__ , [ ] ) end @impl true def init ( [ ] ) do { :ok , nil , connection_name : :gnat , stream_name : &quot;HELLO&quot; , consumer_name : &quot;LOGGER&quot; } end @impl true def handle_message ( message , state ) do IO . inspect ( message ) { :ack , state } end end Pull Consumer is a regular GenServer and it takes a reference to Gnat.ConnectionSupervisor along with names of Jetstream stream and consumer as options passed to Jetstream.PullConsumer.start* functions. These options are passed as keyword list in third element of tuple returned from the Jetstream.PullConsumer.init/1 callback. The only required callbacks are well known gen server's Jetstream.PullConsumer.init/1 and Jetstream.PullConsumer.handle_message/2 , which takes new message as its first argument and is expected to return an ACK action instructing underlying process loop what to do with this message. Here we are asking it to automatically send for us an ACK message back to Jetstream. Let's now create a consumer in our NATS server. We will call it LOGGER as we plan to let it simply log everything published to the stream. nats consumer add --pull --deliver=all HELLO LOGGER Now, let's start our pull consumer under application's supervision tree. def start ( _type , _args ) do children = [ ... # Jetstream Pull Consumer HelloJetstream.LoggerPullConsumer , ] ... Let's now publish some messages to our HELLO stream, so something will be waiting for our application to be read when it starts.","ref":"getting_started.html#creating-a-pull-consumer","title":"Getting Started - Creating a pull consumer","type":"extras"},{"doc":"Jetstream listens on regular NATS subjects, so publishing messages is dead simple with Gnat.pub/3 : Gnat . pub ( :gnat , &quot;greetings&quot; , &quot;Hello World&quot; ) Or via NATS CLI: nats pub greetings &quot;Hello World&quot; That's it! When you run your app, you should see your messages being read by your application.","ref":"getting_started.html#publising-messages-to-streams","title":"Getting Started - Publising messages to streams","type":"extras"},{"doc":"Jetstream provides a JSON API for managing streams and consumers. This library exposes this API via interactions with the Jetstream.Api.Stream and Jetstream.Api.Consumer modules. These modules act as native wrappers for the API and do not attempt to simplify any of the common use-cases. As this library matures we may introduce a separate layer of functions to handle these scenarios, but for now our aim is to provide full access to the Jetstream API.","ref":"managing.html","title":"Managing Streams and Consumers","type":"extras"},{"doc":"# Start a nats server with jetstream enabled and default configs # Now run the following snippets in an IEx terminal alias Jetstream.API . { Consumer , Stream } # Setup a connection to the nats server and create the stream/consumer # This is the equivalent of these two nats cli commands # nats stream add TEST --subjects=&quot;greetings&quot; --max-msgs=-1 --max-msg-size=-1 --max-bytes=-1 --max-age=-1 --storage=file --retention=limits --discard=old # nats consumer add TEST TEST --target consumer.greetings --replay instant --deliver=all --ack all --wait=5s --filter=&quot;&quot; --max-deliver=10 { :ok , connection } = Gnat . start_link ( ) stream = % Stream { name : &quot;TEST&quot; , subjects : [ &quot;greetings&quot; ] } { :ok , _response } = Stream . create ( connection , stream ) consumer = % Consumer { stream_name : &quot;TEST&quot; , name : &quot;TEST&quot; , deliver_subject : &quot;consumer.greetings&quot; , ack_wait : 5_000_000_000 , max_deliver : 10 } { :ok , _response } = Consumer . create ( connection , consumer ) # Setup Consuming Function defmodule Subscriber do def handle ( msg ) do IO . inspect ( msg ) case msg . body do &quot;hola&quot; -&gt; Jetstream . ack ( msg ) &quot;bom dia&quot; -&gt; Jetstream . nack ( msg ) _ -&gt; nil end end end # normally you would add the `ConnectionSupervisor` and `ConsumerSupervisor` to your supervisrion tree # here we start them up manually in an IEx session { :ok , _pid } = Gnat.ConnectionSupervisor . start_link ( %{ name : :gnat , backoff_period : 4_000 , connection_settings : [ %{ } ] } ) { :ok , _pid } = Gnat.ConsumerSupervisor . start_link ( %{ connection_name : :gnat , consuming_function : { Subscriber , :handle } , subscription_topics : [ %{ topic : &quot;consumer.greetings&quot; } ] } ) # now publish some messages into the stream Gnat . pub ( :gnat , &quot;greetings&quot; , &quot;hello&quot; ) # no ack will be sent back, so you&#39;ll see this message received 10 times with a 5sec pause between each one Gnat . pub ( :gnat , &quot;greetings&quot; , &quot;hola&quot; ) # an ack is sent back so this will only be received once Gnat . pub ( :gnat , &quot;greetings&quot; , &quot;bom dia&quot; ) # a -NAK is sent back so you&#39;ll see this received 10 times very quickly","ref":"push_based_consumer.html","title":"Push based consumer","type":"extras"},{"doc":"Broadway is a library which allows building concurrent and multi-stage data ingestion and data processing pipelines with Elixir easily. You can learn about it more in Broadway documentation . Jetstream library comes with tools necessary to use NATS Jetstream with Broadway.","ref":"broadway.html","title":"Using Broadway with Jetstream","type":"extras"},{"doc":"In order to use Broadway with NATS Jetstream you need to: Setup a NATS Server with JetStream turned on Create stream and consumer on NATS server Configure Gnat connection in your Elixir project Configure your project to use Broadway In this guide, we are going to focus on the fourth point. To learn how to start Jetstream locally with Docker Compose and then add Gnat and Jetstream to your application, see the Starting Jetstream section in Getting Started guide . Adding Broadway to your application Once we have NATS with JetStream running and the stream and consumer we are going to use are created, we can proceed to adding Broadway to our project. First, put :broadway to the list of dependencies in mix.exs . defp deps do [ ... { :broadway , ... version ... } , ... ] end Visit Broadway page on Hex.pm to check for current version to put in deps . To install the dependencies, run: mix deps.get Defining the pipeline configuration The next step is to define your Broadway module. We need to implement three functions in order to define a Broadway pipeline: start_link/1 , handle_message/3 and handle_batch/4 . Let's create start_link/1 first: defmodule MyBroadway do use Broadway alias Broadway.Message def start_link ( _opts ) do Broadway . start_link ( __MODULE__ , name : MyBroadwayExample , producer : [ module : { OffBroadway.Jetstream.Producer , connection_name : :gnat , stream_name : &quot;TEST_STREAM&quot; , consumer_name : &quot;TEST_CONSUMER&quot; } , concurrency : 10 ] , processors : [ default : [ concurrency : 10 ] ] , batchers : [ default : [ concurrency : 5 , batch_size : 10 , batch_timeout : 2_000 ] ] ... ) end ... callbacks .. end All start_link/1 does is just delegating to Broadway.start_link/2 . To understand what all these options mean and to learn about other possible settings, visit Broadway documentation . The part that interests us the most in this guide is the producer.module . Here we're choosing OffBroadway.Jetstream.Producer as the producer module and passing the connection options, such as Gnat process name and stream name. For full list of available options, visit Producer documentation. Implementing Broadway callbacks Broadway requires some callbacks to be implemented in order to process messages. For full list of available callbacks visit Broadway documentation . A simple example: defmodule MyBroadway do use Broadway alias Broadway.Message ... start_link ... def handle_message ( _processor_name , message , _context ) do message |&gt; Message . update_data ( &amp; process_data / 1 ) |&gt; case do &quot;FOO&quot; -&gt; Message . configure_ack ( on_success : :term ) &quot;BAR&quot; -&gt; Message . configure_ack ( on_success : :nack ) message -&gt; message end end defp process_data ( data ) do String . upcase ( data ) end def handle_batch ( _ , messages , _ , _ ) do list = messages |&gt; Enum . map ( fn e -&gt; e . data end ) IO . puts ( &quot;Got a batch: \#{ inspect ( list ) } . Sending acknowledgements...&quot; ) messages end First, in handle_message/3 we update our messages' data individually by converting them to uppercase. Then, in the same callback, we're changing the success ack option of the message to :term if its content is &quot;FOO&quot; or to :nack if the message is &quot;BAR&quot; . In the end we print each batch in handle_batch/4 . It's not quite useful but should be enough for this guide.","ref":"broadway.html#getting-started","title":"Using Broadway with Jetstream - Getting started","type":"extras"},{"doc":"Once we have our pipeline fully defined, we need to add it as a child in the supervision tree. Most applications have a supervision tree defined at lib/my_app/application.ex . children = [ { MyBroadway , [ ] } ] Supervisor . start_link ( children , strategy : :one_for_one ) You can now test the pipeline. Let's start the application: iex -S mix Use Gnat API to send messages to your stream: Gnat . pub ( :gnat , &quot;test_subject&quot; , &quot;foo&quot; ) Gnat . pub ( :gnat , &quot;test_subject&quot; , &quot;bar&quot; ) Gnat . pub ( :gnat , &quot;test_subject&quot; , &quot;baz&quot; ) Batcher should then print: Got a batch : [ &quot;FOO&quot; , &quot;BAR&quot; , &quot;BAZ&quot; ] . Sending acknowledgements ...","ref":"broadway.html#running-the-broadway-pipeline","title":"Using Broadway with Jetstream - Running the Broadway pipeline","type":"extras"}]